@@ -176,8 +176,8 @@ const µBlock = {  // jshint ignore:line
 
     // Read-only
     systemSettings: {
-        compiledMagic: 47,  // Increase when compiled format changes
-        selfieMagic: 47,    // Increase when selfie format changes
+        compiledMagic: 55,  // Increase when compiled format changes
+        selfieMagic: 55,    // Increase when selfie format changes
     },
 
     // https://github.com/uBlockOrigin/uBlock-issues/issues/759#issuecomment-546654501
@@ -211,6 +211,9 @@ const µBlock = {  // jshint ignore:line
     availableFilterLists: {},
     badLists: new Map(),
 
+    inMemoryFilters: [],
+    inMemoryFiltersCompiled: '',
+
     // https://github.com/uBlockOrigin/uBlock-issues/issues/974
     //   This can be used to defer filtering decision-making.
     readyToFilter: false,
@@ -270,20 +273,32 @@ const µBlock = {  // jshint ignore:line
         return this;
     }
 
+    maybeFromDocumentURL(documentUrl) {
+        if ( documentUrl === undefined ) { return; }
+        if ( documentUrl.startsWith(this.tabOrigin) ) { return; }
+        this.tabOrigin = originFromURI(µBlock.normalizeTabURL(0, documentUrl));
+        this.tabHostname = hostnameFromURI(this.tabOrigin);
+        this.tabDomain = domainFromHostname(this.tabHostname);
+    }
+
     // https://github.com/uBlockOrigin/uBlock-issues/issues/459
     //   In case of a request for frame and if ever no context is specified,
     //   assume the origin of the context is the same as the request itself.
     fromWebrequestDetails(details) {
         const tabId = details.tabId;
         this.type = details.type;
-        if ( this.itype === this.MAIN_FRAME && tabId > 0 ) {
+        const isMainFrame = this.itype === this.MAIN_FRAME;
+        if ( isMainFrame && tabId > 0 ) {
             µBlock.tabContextManager.push(tabId, details.url);
         }
         this.fromTabId(tabId); // Must be called AFTER tab context management
         this.realm = '';
         this.id = details.requestId;
+        this.setMethod(details.method);
         this.setURL(details.url);
         this.aliasURL = details.aliasURL || undefined;
+        this.redirectURL = undefined;
+        this.filter = undefined;
         if ( this.itype !== this.SUB_FRAME ) {
             this.docId = details.frameId;
             this.frameId = -1;
@@ -293,33 +308,36 @@ const µBlock = {  // jshint ignore:line
         }
         if ( this.tabId > 0 ) {
             if ( this.docId === 0 ) {
+                if ( isMainFrame === false ) {
+                    this.maybeFromDocumentURL(details.documentUrl);
+                }
                 this.docOrigin = this.tabOrigin;
                 this.docHostname = this.tabHostname;
                 this.docDomain = this.tabDomain;
-            } else if ( details.documentUrl !== undefined ) {
-                this.setDocOriginFromURL(details.documentUrl);
-            } else {
-                const pageStore = µBlock.pageStoreFromTabId(this.tabId);
-                const docStore = pageStore && pageStore.getFrameStore(this.docId);
-                if ( docStore ) {
-                    this.setDocOriginFromURL(docStore.rawURL);
-                } else {
-                    this.setDocOrigin(this.tabOrigin);
-                }
+                return this;
             }
-        } else if ( details.documentUrl !== undefined ) {
+            if ( details.documentUrl !== undefined ) {
+                this.setDocOriginFromURL(details.documentUrl);
+                return this;
+            }
+            const pageStore = µBlock.pageStoreFromTabId(this.tabId);
+            const docStore = pageStore && pageStore.getFrameStore(this.docId);
+            if ( docStore ) {
+                this.setDocOriginFromURL(docStore.rawURL);
+            } else {
+                this.setDocOrigin(this.tabOrigin);
+            }
+            return this;
+        }
+        if ( details.documentUrl !== undefined ) {
             const origin = originFromURI(
                 µBlock.normalizeTabURL(0, details.documentUrl)
             );
             this.setDocOrigin(origin).setTabOrigin(origin);
-        } else if ( this.docId === -1 || (this.itype & this.FRAME_ANY) !== 0 ) {
-            const origin = originFromURI(this.url);
-            this.setDocOrigin(origin).setTabOrigin(origin);
-        } else {
-            this.setDocOrigin(this.tabOrigin);
+            return this;
         }
-        this.redirectURL = undefined;
-        this.filter = undefined;
+        const origin = originFromURI(this.url);
+        this.setDocOrigin(origin).setTabOrigin(origin);
         return this;
     }
 
@@ -334,24 +352,31 @@ const µBlock = {  // jshint ignore:line
     }
 
     toLogger() {
-        this.tstamp = Date.now();
-        if ( this.domain === undefined ) {
-            void this.getDomain();
-        }
-        if ( this.docDomain === undefined ) {
-            void this.getDocDomain();
-        }
-        if ( this.tabDomain === undefined ) {
-            void this.getTabDomain();
-        }
-        const filters = this.filter;
+        const details = {
+            id: this.id,
+            tstamp: Date.now(),
+            realm: this.realm,
+            method: this.getMethodName(),
+            type: this.stype,
+            tabId: this.tabId,
+            tabDomain: this.getTabDomain(),
+            tabHostname: this.getTabHostname(),
+            docDomain: this.getDocDomain(),
+            docHostname: this.getDocHostname(),
+            domain: this.getDomain(),
+            hostname: this.getHostname(),
+            url: this.url,
+            aliasURL: this.aliasURL,
+            filter: undefined,
+        };
         // Many filters may have been applied to the current context
-        if ( Array.isArray(filters) === false ) {
-            return logger.writeOne(this);
+        if ( Array.isArray(this.filter) === false ) {
+            details.filter = this.filter;
+            return logger.writeOne(details);
         }
-        for ( const filter of filters ) {
-            this.filter = filter;
-            logger.writeOne(this);
+        for ( const filter of this.filter ) {
+            details.filter = filter;
+            logger.writeOne(details);
         }
     }
 };
