@@ -22,6 +22,8 @@
 
 // For background page or non-background pages
 
+/* global browser */
+
 'use strict';
 
 /******************************************************************************/
@@ -37,7 +39,10 @@ vAPI.setTimeout = vAPI.setTimeout || self.setTimeout.bind(self);
 
 vAPI.webextFlavor = {
     major: 0,
-    soup: new Set()
+    soup: new Set(),
+    get env() {
+        return Array.from(this.soup);
+    }
 };
 
 (( ) => {
@@ -66,19 +71,16 @@ vAPI.webextFlavor = {
         typeof browser.runtime.getBrowserInfo === 'function'
     ) {
         browser.runtime.getBrowserInfo().then(info => {
-            flavor.major = parseInt(info.version, 10) || 60;
+            flavor.major = parseInt(info.version, 10) || flavor.major;
             soup.add(info.vendor.toLowerCase())
                 .add(info.name.toLowerCase());
-            if ( soup.has('firefox') && flavor.major < 57 ) {
-                soup.delete('html_filtering');
-            }
             dispatch();
         });
         if ( browser.runtime.getURL('').startsWith('moz-extension://') ) {
             soup.add('firefox')
                 .add('user_stylesheet')
                 .add('html_filtering');
-            flavor.major = 60;
+            flavor.major = 91;
         }
         return;
     }
@@ -86,11 +88,11 @@ vAPI.webextFlavor = {
     // Synchronous -- order of tests is important
     const match = /\bChrom(?:e|ium)\/([\d.]+)/.exec(ua);
     if ( match !== null ) {
-        soup.add('chromium');
+        soup.add('chromium')
+            .add('user_stylesheet');
         flavor.major = parseInt(match[1], 10) || 0;
-        // https://github.com/gorhill/uBlock/issues/3588
-        if ( flavor.major >= 66 ) {
-            soup.add('user_stylesheet');
+        if ( flavor.major >= 105 ) {
+            soup.add('native_css_has');
         }
     }
 
@@ -115,18 +117,6 @@ vAPI.getURL = browser.runtime.getURL;
 
 /******************************************************************************/
 
-vAPI.i18n = browser.i18n.getMessage;
-
-// http://www.w3.org/International/questions/qa-scripts#directions
-document.body.setAttribute(
-    'dir',
-    ['ar', 'he', 'fa', 'ps', 'ur'].indexOf(vAPI.i18n('@@ui_locale')) !== -1
-        ? 'rtl'
-        : 'ltr'
-);
-
-/******************************************************************************/
-
 // https://github.com/gorhill/uBlock/issues/3057
 // - webNavigation.onCreatedNavigationTarget become broken on Firefox when we
 //   try to make the popup panel close itself using the original
